@@ -38,12 +38,20 @@ public:
 
     int count();
     QMap<int, QList<DeviceInfo>> getAllDevices();
+    QMap<int, QList<DeviceInfo>> getUserDevices(int uid);
     QList<DeviceInfo> getDevices(int type);
     DeviceInfo* getDefaultDevice(uid_t uid);
     DeviceInfo* findDevice(const QString &deviceName);
+    DeviceInfo* findDevice(const int id);
     DeviceInfo* getFirstDevice();
     int getFeatureCount(int uid, int indexStart = 0, int indexEnd = -1);
     static QString bioTypeToString_tr(int type);
+    void setIsShowHotPlug(bool isShow);
+    int GetUserDevFeatureCount(int uid,int drvid);
+    int GetUserDevCount(int uid);
+    bool getUseFirstDevice();
+    int getFailedTimes();
+    bool GetHiddenSwitchButton();
 
 private:
     void connectToService();
@@ -58,6 +66,8 @@ private slots:
 private:
     QDBusInterface                  *serviceInterface;
     QList<DeviceInfo*>               deviceInfos;        //the list of al device info
+    bool                             isShowHotPlug;
+    bool                        useFirstDevice;
 };
 
 
